#! /usr/bin/python

"""
     By: Jim Kemp
     Powerhouse Electronics 
     ph-elec.com
     10/26/2009

   This application interfaces to a Melexis I/O board. The I/O board sends two raw 
 temperatures every 200ms. The raw temperatures come as a six byte packet. Each 
 temperature reading is 16bits plus an eight bit CRC-8 (Melexis calls it a PEC ->
 packet error code. See the code in myStreamReader.py for more info.

    The Gui part of this application simply plots the data onto a graph. The app also
 allows a slider to control the SSR precent output.
"""

import sys
import numpy as np
import matplotlib 
matplotlib.use('GTK')

from matplotlib.figure import Figure
from matplotlib.axes import Subplot
from matplotlib.backends.backend_gtk import FigureCanvasGTK, NavigationToolbar
from matplotlib.numerix import arange, sin, pi
from matplotlib.ticker import MultipleLocator, FormatStrFormatter

try:
    import pygtk, gobject
    pygtk.require("2.0") 
except:
    print "no pygtk"
    pass
try:
    import gtk, gobject
    import gtk.glade
except:
    print "Error, import of Gtk/Gtk.Glade failed."
    sys.exit(1)

import serial
import threading, Queue
from time import sleep

# myStreamReader handles all the details of reading from the sensor.
from myStreamReader import *

sleep( 0.05 )

SCR_UPDATE_RATE         = 2000  # (units ms)
PROCESS_DATA            = 2000  # (units ms)
SER_RATE                = 100    # (units ms)

scaleX                  = 0.2  # sec / DataPt
widthX                  = 2    # Starting width of X axis, in minutes.
maxWidthX               = 20    # Maximum X axis time, in minutes.

# Buffer Len of 10 Sec = 5 reads/sec * 10sec
LEN = int( 10 / scaleX )  # Rate Calculation Width
MED = int( 5  / scaleX )  # Median Filter Length (default = 5min)

Qtemps = Queue.Queue()  # Define a queue to handle fifo buffering of temperature readings.

# =============================================================================
class appGui( ): 

    #--------------------------------------------------------------------------
    def __init__( self ):
        self.updateAxis = True
        self.i = 0
        self.running = True
        self.yMax = 100
        self.yMin = 0
        self.zeroPt = 0
        self.cntPts = 0
        self.CntPtsNum = 0
        self.CntPtsOld = 0
        self.ssrCnt = 0      # SSR Threshold Count -> 1 Count = 0.2s
        self.ssrMax = 20     # SSR cycle rate      -> 20 * 200ms = 4 seconds.
        self.rate = 0        # Temperature rate of change (degC / second)
        self.widthX = widthX

        gladefile = "IR_Gui.glade" 
        self.windowname = "window1" 
        self.wTree = gtk.glade.XML( gladefile, self.windowname ) 
        self.window = self.wTree.get_widget( "window1" )

        dic = { "on_window1_destroy" : self.on_bnExit_clicked,
                "on_bnExit_clicked" : self.on_bnExit_clicked,
                "on_window1_check_resize" : self.on_Resize,
                "on_chkLogging_toggled" : self.on_Logging_Toggle,
                "on_btnClear_clicked" : self.on_btnClear_clicked,
                "on_btnScaleY_clicked" : self.on_btnScaleY_clicked
                }

        self.wTree.signal_autoconnect( dic )

        # setup matplotlib stuff on first notebook page (empty graph)
        self.figure = Figure(figsize=(6,4), dpi=72)
        self.axis = self.figure.add_subplot(111) 
        self.line, self.line2 = self.axis.plot( ([0],[0]), 'b-', ([0], [0]), 'k-' )

        # Setup tick marks on X & Y axis.
        self.axis.xaxis.set_major_locator( MultipleLocator(1) )
        self.axis.xaxis.set_major_formatter( FormatStrFormatter('%d') )
        self.axis.xaxis.set_minor_locator(  MultipleLocator( 0.5 ) )
        self.axis.xaxis.grid( True, which='minor' )  # minor grid on too
        self.axis.yaxis.set_minor_locator(  MultipleLocator( 5 ) )
        self.axis.yaxis.grid( True, which='minor' )  # minor grid on too
        self.axis.grid( True )

        # Setup line widths and line priority.
        self.line.set_linewidth( 2.0 )
        self.line.set_zorder( 2 )
        self.line2.set_linewidth( 2.0 )
        self.line2.set_zorder( 1 )

        # Setup axis scales.
        self.axis.set_ylim( ymax = self.yMax, ymin = self.yMin )
        self.axis.set_xlim( xmax = widthX * 60, xmin = 0 )
        self.axis.set_xbound( 0, int(widthX) )

        # Clear the buffers.
        self.xs = [] ; self.ys = [] ; self.y = [] ; self.i = 1
        # Clear the buffers and set the empty lists into the chart. 
        # Clearing the buffers dumps the dumb zero used to build the chart.
        self.line.set_data( self.xs, self.ys )
        self.line2.set_data( self.xs, self.y )

        self.axis.set_xlabel( 'Time (minutes)' ) 
        self.axis.set_ylabel( 'Temperature (DegC)' )
        self.axis.set_title( 'IR Profile' )
        self.axis.grid( True )
        self.canvas = FigureCanvasGTK( self.figure ) # a gtk.DrawingArea 

        self.canvas.show() 
        self.graphview = self.wTree.get_widget( "vbox1" )
        self.graphview.pack_start( self.canvas, True, True )
        self.window.show_all()

        self.figure.canvas.draw()

        self.xs = list( self.line.get_xdata() )
        self.ys = list( self.line.get_ydata() )
        self.y = list( self.line2.get_ydata() )

        # Bang myReader until it gets synced with the serial stream!
        i = 0
        while i < 100:
            Tobj, Tamb = myReader( 0 )
            if Tobj != 0: break;
        self.T = [Tobj] * LEN   # Init a list of saved temperature readings.
        self.Tmedian = [Tobj] * MED

        # Setup repeating timers.
        self.timeoutId = gobject.timeout_add( SCR_UPDATE_RATE, self.updateScreen )
        self.timeoutId2 = gobject.timeout_add( PROCESS_DATA, self.processData )
        self.timeoutId2 = gobject.timeout_add( SER_RATE, self.streamReader )

        # Get handles to the Glade GUI objects.
        self.lbPtrPerSec = self.wTree.get_widget( "lbPtsPerSec" )
        self.lbValues = self.wTree.get_widget( "lbValues" )
        self.cbAutoMode = self.wTree.get_widget( "cbAutoMode" )
        self.svOutput = self.wTree.get_widget( "vsOutput" )
        self.svOutput.set_value( 0 ) # Set default slider output to zero.


    # Dump all the plot data and clear the plot.
    #--------------------------------------------------------------------------
    def on_btnClear_clicked ( self, btnClear ):
        # Clear the data lists.
        self.xs = []
        self.ys = []
        self.y = []
        self.i = 1

        # Set the scales back to defaults.
        self.axis.set_ylim( ymax = self.yMax, ymin = self.yMin )
        self.axis.set_xlim( xmax = widthX * 60, xmin = 0 )
        self.axis.set_xbound( 0, int(widthX) )

        # Set the empty lists into the chart.
        self.line.set_data( self.xs, self.ys )
        self.line2.set_data( self.xs, self.y )

        # Reset the counters.
        self.CntPtsNum = 0
        self.cntPts = 0


    # Find the max value, add 10C and set the scale.
    #--------------------------------------------------------------------------
    def on_btnScaleY_clicked ( self, btnScaleY ):
        y1 = list( self.line.get_ydata() )   # Get a list of temp from the chart.
        y2 = list( self.line2.get_ydata() )  # Get from both lines.

        # Add the two list together, find the max value, and add 10 to it.
        # I love Python!
        ym = max( y1 + y2 ) + 10

        self.axis.set_ylim( ymax = ym, ymin = 0 )
        self.figure.canvas.draw()

        self.yMax = ym
        self.yMin = 0

    # This is not really implemented. Really, it should be changed to a button to
    # "Export Data". The plot data could just be written to a file. Simple.
    #--------------------------------------------------------------------------
    def on_Logging_Toggle ( self, chkBtn ):
        if chkBtn.get_active():
            chkBtn.set_label( "Logging On" )
            #ser.set_Logging( True )
        else:
            chkBtn.set_label( "Logging Off" )
            #ser.set_Logging( False )

    #--------------------------------------------------------------------------
    def on_Resize ( self, *args ):
        print "on_Rezise"

    #--------------------------------------------------------------------------
    def on_bnExit_clicked ( self, *args ):
        print "bnExit"
        self.running = False    # Flag to the thread we want to end.
        gtk.main_quit()         # Stop Gtk main loop.


    # Handle the stream of data from the sensor here.
    #--------------------------------------------------------------------------
    def streamReader( self ):
        global ser
        maxQ = 100

        # As long as there are room in the queue push new temps as they become available.
        if (Qtemps.qsize() < maxQ) and self.running:
            ssr = self.svOutput.get_value() # Get Gui slider value.

            # Make a call into myStreamReader - this blocks until a readings are available.
            Tobj, Tamb = myReader( ssr )    # Return object and ambient temperatures.

            if (Tobj != 0) and (Tamb != 0):
                # Ignore dumb values. (Should be using CRC).
                if (Tobj > 300) or (Tobj < 10):
                    return True
                if (Tamb > 200) or (Tamb < 10):
                    return True

                # Median Filter the object temperature...
                # -----------------------------------------------
                # Dump oldest value / Add new value.
                del self.Tmedian[0] ; self.Tmedian.append( Tobj ) 
                tmp = [] ; tmp.extend(self.Tmedian) # Make a copy 
                tmp.sort() # Sort the copy
                Tobj = tmp[ MED / 2 ] # Pick the median (middle) value.

                # Calulate rate of change...
                # -----------------------------------------------
                # Dump the oldest and append new.
                del self.T[0] ; self.T.append( Tobj ) 
                # Calulate rate across 10 seconds.
                rate = (self.T[LEN-1] - self.T[0]) / (LEN * scaleX)      

                # Print results to Gui label.
                self.lbValues.set_label( 
                    'Obj:\t\t%.1f degC\nAmb:\t%.1f degC\nRate:\t%.2f degC/s' % \
                   (Tobj, Tamb, rate) ) 
                
                # Push the two temperatures into a fifo queue.
                Qtemps.put( (Tobj, Tamb) )

        if Qtemps.qsize() >= maxQ:
            print "Qtemps Full!"

        # Return true so gobject will call on us again.
        if self.running:
            return True
        else:
            return False

    # At a slower rate, pull all the queued temperatures at plot them on the 
    # graph.
    #--------------------------------------------------------------------------
    def processData ( self ):

        plotWidth = widthX / scaleX * 60.0

        # Ckeck for, and process, all new data from the fifo queue.
        while Qtemps.qsize( ):
            # Pop temperatures out of the queue as pairs of numbers.
            Tobj, Tamb = Qtemps.get( 0 ) # The zero argument means don't block.
            self.cntPts += 1

            # Capture/remember any new maximum values.
            if Tobj > self.yMax: self.yMax = Tobj
            if Tobj < self.yMin: self.yMin = Tobj

            # Append the new data onto the growing lists.
            self.xs.append( self.i * scaleX / 60.0 )
            self.ys.append( Tobj )
            self.y.append( Tamb )

            self.i += 1
            xl, xh = self.axis.get_xbound()
            maxX = int( xh / scaleX * 60)       # Maximum X axis right hand value.

            # Check if we need to expend the X axis to hold more data.
            if (self.i > maxX) and ( self.i < maxWidthX * 60 / scaleX ):
                self.axis.set_xbound( 0, xh+1 ) # Move the X axis by one minute.

            # Check if X axis is sooooo big that it needs to be shifted (and clipped).
            elif self.i > xh / scaleX * 60:
                # Move X axis by one minute.
                self.axis.set_xbound( xl+1, xh+1 ) # Move the X axis by one minute.
                oneMin = int( 60 / scaleX )        
                del self.xs[:oneMin]               # Purge one minute's worth of old data.
                del self.ys[:oneMin]
                del self.y[:oneMin]

            # Set the data to the plot.
            self.line.set_data( self.xs, self.ys )
            self.line2.set_data( self.xs, self.y )

        if self.running:
            return True
        else:
            return False

    #--------------------------------------------------------------------------
    def updateScreen ( self ):

        #self.line.set_data( self.xs, self.ys )

        if self.updateAxis :
            locator = self.axis.yaxis.get_major_locator()
            vmin, vmax = locator.autoscale()
            if (self.yMax != vmax) or (self.yMin != vmin):
                self.axis.set_ylim( ymax = self.yMax, ymin = self.yMin )

            self.figure.canvas.draw()
        else:
            self.line.figure.canvas.draw()

        self.CntPtsNum += 1
        ptsAvg = (self.cntPts / self.CntPtsNum) / 2.0
        if self.cntPts == self.CntPtsOld:
            self.CntPtsNum = 0
            self.cntPts = 0
        self.CntPtsOld = self.cntPts

        s = '<span font_desc="Arial Bold 10">%.1f pts/sec</span>' % ptsAvg
        self.lbPtrPerSec.set_label( s )

        if self.running:
            return True
        else:
            return False

    #--------------------------------------------------------------------------
    def endGui ( self ):
        # Perform any other cleanup here...
        self.running = False
        closeReader()
        print "End Gui..."
        


# End of Class: appGui
# =============================================================================



# Main Loop....
# =============================================================================

a = appGui()
gtk.main()    # Loop until "gtk.main_quit()" is called.

a.endGui()    # Last chance cleanup.

print; print "End..."



