#! /usr/bin/python

from time import sleep
import serial
from time import sleep

"""
=============================================================================
IR Temperature Controller - Stream Reader: 

     By: Jim Kemp
     Powerhouse Electronics 
     ph-elec.com
     10/26/2009

This Python code reads the RS232 stream from a PIC-12F609 micro. The micro
also has an interface to a Melexis IR temperature sensors.

The Melexis MLX90614 is an infrared temperature sensor. This Python "host"
app interfaces to a PIC-12F609 micro which does the actual SMBUS talking
to the sensor. The PIC also accepts an ASCII '0' or '1' to control a 
digital output. The output can be used to control a SSR (solid state relay)
like a Crydom. A simple closed loop temperature control system can be built
using the temperature input and the SSR output.

Melexis Sensor Used -> Digikey Part Number: MLX90614ESF-BAA-ND

PIC 12F609 Packet Deception:
     1          2         3         4         5         6       <- Byte Index
 -------------------------------------------------------------
| Tobj HB | Tobj LB | Tobj PEC | Tamb HB | Tamb LB | Tamb PEC | <- Description
 -------------------------------------------------------------
Repeats approximately every 200ms with new readings. 
HB = High Byte (MSB)
LB = Low Byte (LSB)

Tobj = IR Temperature of Object
Tamb = Sensor Ambient Temperature
PEC = Packet Error Code (CRC8)
See Melexis datasheet for more info.

The 12F609 takes 10ms to read the sensor and post the packet to via RS232 
9600 baud the host. During the 10ms read/post time the micro does not respond
to commands. To avoid sending during this deaf period, simply wait until
a packet is received, then dwell for another 50ms before sending a 
SSR command. In this way, the SSR command is sent when the Pic is listening.
A good SSR command will be echoed back as an ACK. A bad SSR command will be 
NACKed (0x00).

A SSR command is a single byte of ASCII as either '1' or '0''. Any
other character will be NACKed.

If, after a SSR command, there is no ACK or NACK the command probably 
was sent during the 10ms sensor processing time. No ACK means the SSR command 
was ignored. 

=============================================================================
"""

# Windows Users: simple change the first argument from '/dev/ttyUSB0' to a
# COM port number (minus one). Example, to open COM4, set first argument to 3.

# The micro sends a packet approx every 200ms. A timeout of  50ms gets us 
# aligned to the burst of bytes in the packet.
ser = serial.Serial( '/dev/ttyUSB0', 9600, timeout=0.05 )
ser.flushInput()

# The SSR output is a PWM output. The cycle time is approx 4 seconds. The 
# duty cycle has a resolution of 200ms. So, we have 1 part in 20 control.

ssrCnt = 0      # SSR Threshold Count -> 1 Count = 0.2s
ssrMax = 20     # SSR cycle rate      -> 20 * 200ms = 4 seconds.
nackCnt = 0     # Count the number of NACKs.
oldSsr = 0
needAck = 0

# ==============================================================================
def myReader( ssr ):
    """
    This function should be called often 100ms. The function will block up to
    another 50ms waiting for new data is read from the serial port. Note, the 
    serial has an argument to set the timeout. After a packet is read, the 
    SSR command is sent out. This keeps the output updated on the micro.

    The temperature readings are returned as a tuple.
    """
    global ssrCnt, ssrMax, nackCnt, oldSsr, needAck
    cTobj, cTamb = 0, 0

    # Get a packet - Wait upto 50ms for the packet.
    str = ser.read( 6 )

    # If we have a full packet of six byte then process it...
    if len(str) == 6:

        S = []
        # Convert string of characters into bytes.
        for s in str: S.append( ord(s) )

        # Convert IR object raw temp into Celsius and Fahrenheit
        cTobj = ((S[0] << 8 ) | S[1]) * 0.02 - 273.15
        #fTobj = (cTobj * 9.0) / 5.0 + 32.0
        # Should do a crc calculation here.

        # The sensor also knows it's own die temperature.
        cTamb = ((ord(str[3]) << 8 ) | ord(str[4])) * 0.02 - 273.15
        #fTamb = (cTamb * 9.0) / 5.0 + 32.0
        # Should do a crc calculation here.

        # Let ssrCnt range from 0 up to ssrMax-1.
        ssrCnt = (ssrCnt + 1) % ssrMax
        #print ssr, ssrCnt, ssrMax

        # Wait 20ms after a read before trying to send a SSR command to the Pic.
        sleep( 0.02 )

        # Convert ssr (a percentage) into a threshold. If threshold
        # is greater than loop count, then turn on output.
        if (ssrMax * (ssr / 100.0)) > ssrCnt: 
            if oldSsr!= 1:
                ser.write( '1' )
                oldSsr = 1
                needAck = 1
        else:                                 
            if oldSsr != 0:
                ser.write( '0' )
                oldSsr = 0
                needAck = 1

        if needAck:
            # Now get Ack/Nack from micro.
            ack = ser.read( 1 )
            if (ack == '1') or (ack == '0'): 
                pass
            else:             
                cTobj, cTamb = 0, 0
                print "NACK:", nackCnt
                nackCnt += 1
                ser.flushInput()
            needAck = 0


    # Return the two measured temperatures as a tuple.
    return cTobj, cTamb

# ==============================================================================
def closeReader():
    # Make sure the SSR gets turned off.
    i = 0
    while ++i < 10:
        ser.write( '0' )
        ack = ser.read( 1 )
        if (ack == '1') or (ack == '0'): break
    if i >= 10: print "Error turning off SSR."
    ser.close()





# Perform some testing to make sure she works!
# ==============================================================================
if __name__ == '__main__':

    while 1:
        # Set the SSR output to 1%. # SSR should come on for 50ms every 4 seconds.
        Tobj, Tamb = myReader( 1 )
        print "Obj: %.1f\tAmb: %.1f" % ( Tobj, Tamb )
        sleep( 0.1 )


